/* $Workfile: OtxType.h $   $Revision: 74 $    $Modtime: 12/23/03 5:10p $  */
#ifndef OTX_TYPE_H
#define OTX_TYPE_H
/*******************************************************************************

  Copyright (C) 1998 - 2003 Odin TeleSystems Inc., All Rights Reserved.

  @doc PublicHwApi OtxType

  OTX Generic Hardware Driver API

  @module
  OtxType | Types used in OTX driver API

  @contents2
  OtxType.h |

  @index const enum struct type func | OtxType

  @contents2
  OTX Driver Level Types |

  @head3 Enumerated Types  |

  @index enum | OtxType

  @head3 Struct Types |

  @index struct | OtxType

  @head3 Other Types |

  @index type | OtxType

*******************************************************************************/

#include "OtxDef.h"

// Fix packing for IO Control Structs
#if defined(__linux__)
#pragma pack(4)
#elif defined(__BORLANDC__)
#pragma option push -a4
#else
#pragma pack(push,4)
#endif

/*******************************************************************************
  Basic Data Types
*******************************************************************************/

// @type OTX_INT8 | Signed integer, 8 bits
typedef OsaInt8   OTX_INT8;
// @type OTX_INT16 | Signed integer, 16 bits
typedef OsaInt16  OTX_INT16;
// @type OTX_INT32 | Signed integer, 32 bits
typedef OsaInt32  OTX_INT32;

// @type OTX_UINT8 | Unsigned integer, 8 bits
typedef OsaUint8  OTX_UINT8;
// @type OTX_UINT16 | Unsigned integer, 16 bits
typedef OsaUint16 OTX_UINT16;
// @type OTX_UINT32 | Unsigned integer, 32 bits
typedef OsaUint32 OTX_UINT32;

// @type OTX_FP64 | Floating point value, 64 bits precision
typedef OsaFp64 OTX_FP64;

// @type OTX_INT64 | struct with 64 bits. Organized as a union to simplify access to the two 32 bit parts
// The bits in the structs are organized so that it can be treated as a 64 bit int using compiler support
// for calculations when available, see also OsaInt64.
typedef OsaInt64          OTX_INT64;

// @type OTX_UINT64 | struct with 64 bits. Organized as a union to simplify access to the two 32 bit parts
// The bits in the structs are organized so that it can be treated as a 64 bit unsigned int using compiler
// support for calculations when available, see also OsaUint64.
typedef OsaUint64         OTX_UINT64;

// @type OTX_CHAR | Character type (8 bits)
typedef char OTX_CHAR;

// @type OTX_PVOID | Void Pointer type (32 bits)
typedef OsaVoidP OTX_PVOID;

// @struct OTX Driver Version Struct
struct OtxVersionInfo {
  OTX_UINT16  nMjRev;
  OTX_UINT16  nMnRev;
  OTX_UINT16  bRelStatus;
  OTX_UINT16  nPrelLevel;
};

typedef struct OtxVersionInfo OtxVersionInfo;

// Use to elliminate warnings about unreferenced parameters
#define OTX_UNREF(x) x = x

/*******************************************************************************
  Device Handles
*******************************************************************************/

/*******************************************************************************

     @type OTX_HANDLE | Standard handle to Physical, Logical, Pipe, and Event Queue Devices

     @ex OTX_HANDLE bit values layed out as follows |
       Only positive values are valid handles.
       Negative values are not valid. Especially, -1 denotes a invalid handle.
    
         Bits  0-27: contain the handle value
         Bit     28: Least significant bit of device type field
         Bit     29: Intermediate bit of device type field
         Bit     30: Most significant bit of device type field
         Bit     31: Must be '0'

         |3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0
         |1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
         |-+-+-+-+-+-+-+-+-----------------------------------------------+
         |0|D|D|D| | | | |                  HANDLE                       |
         |-+-+-+-+-+-+-+-+-----------------------------------------------+
      
         DDD =
           '001; then handle to a Physical Device
           '010; then handle to a Logical Device
           '011; then handle to a Event Queue Device
           '100; then handle to a Pipe Device
           '101; then handle to a Service
           '110; then handle to a Call
      
*******************************************************************************/
typedef OTX_UINT32  OTX_HANDLE;

// @type OTX_EVENT | Handle to an Event, OTX_EVENT is treated differently from OTX_HANDLE.
#if defined(_WIN32_WCE)
typedef int OTX_EVENT;
#else
typedef void *OTX_EVENT;
#endif

/*******************************************************************************
  Driver Data Types
*******************************************************************************/

// @type OTX_DATETIME | Absolute time in 100 nanosecond increments since midnight Jan 1, 1601, often used for time stamps.
typedef OsaDateTime OTX_DATETIME;

// @type OTX_TIME | Relative time in milliseconds, often used for timeouts.
typedef OTX_UINT32 OTX_TIME;

// @type OTX_RESULT | Standard result code returned by all the OTX API functions
typedef OTX_INT32  OTX_RESULT;


// @enum OTX_BOOL | OTX Specific Boolean Data Type
// @emem OTX_TRUE | True value
// @emem OTX_FALSE | False Value
typedef OsaBool32   OTX_BOOL;
#define OTX_TRUE    OSA_TRUE
#define OTX_FALSE   OSA_FALSE


// @type OTX_TASK_REF | Used for function calls that can not always be completed without
// waiting for device operations. If the task remains pending after the function has
// returned (usually with OTX_E_PENDING) a notification will be received in the
// associated event queue when the operation completes. The notification contains the
// reference value that was given in the function call. Note that value '0' is special.
//
// A pending task can be cancelled. by the user with <f OtxCancelTask>.
// Also the task is automatically cancelled at the end of the specified timeout period.
// If not completed or manually cancelled before. Cancelled tasks result in notifications
// queued in the associated event queue.
// Note that a task is always associated with a device and a device is may be associated
// with an event queue. The task is associated with its associated device's event queue.
//
// @flag '0' | Blocking call, tries to complete operation before given timeout. Operation
// completed or cancelled when the function returns. No notifications ever queued in
// the associated event queue.
//
// @flag >='1' | Non-blocking call. Function returns immediately either with the result
// of the requested operation or with task left as pending, in which case a notification is
// queued in the associated event queue when the operation completes. The notification
// contains the associated task reference so the user knows which operation completed.
// If the user do not care about keeping track of when particular tasks are completed,
// it is possible to always pass a task reference with the same non-zero value.
typedef OTX_INT32 OTX_TASK_REF;

// @type OTX_NOTIFY_CAUSE | Every notification read from an event queue holds
// a code describing the reason for the notification
// @todo OTX_NOTIFY_CAUSE should maybe be an enum type
typedef OTX_INT32 OTX_NOTIFY_CAUSE;

// @const Used to communicate with the static driver device. E.g. when reading the Revision attribute from the driver
#define OTX_DRIVER_DEVICE_HANDLE  ((OTX_HANDLE) 0)            
// @const Used to communicate with the library. E.g. when reading the Revision attribute from the library
#define OTX_LIB_HANDLE  ((OTX_HANDLE) -2)            
// @const Used to mark an invalid Handle to Physical and Logical devices. Used as type OTX_HANDLE
#define OTX_INVALID_HANDLE_VALUE  ((OTX_HANDLE) -1)            
// @const Used to mark an invalid Event
#define OTX_INVALID_EVENT_VALUE   ((OTX_EVENT) -1)              
// @const Used to mark time infinite time; E.g., timeout that will never expire. Used as type OTX_TIME
#define OTX_TIME_INFINITY         ((OTX_TIME) -1)          
// @const The only task ref value that causes blocking
#define OTX_TASK_SYNC             ((OTX_TASK_REF) 0)
// @const Arbitrary don't care task ref value (avoid using as normal task ref)
#define OTX_TASK_ASYNC            ((OTX_TASK_REF) -1)

// @const Received in events that are unreleated to any requested task
// Do not use as a task ref in API calls
#define OTX_TASK_NONE             ((OTX_TASK_REF) 0)

// @type OTX_ITERATOR |
//
// Opaque data type used to store position information when enumerating
// (iterating over) e.g. attributes of a driver object. The value should never
// be interpreted and the type should never be assumed to be of a certain size. 
// An OTX_ITERATOR can never be directly set to any value.
// instead an OTX_ITERATOR is initialized using <f OtxDrvIteratorBegin>,
// and checked for if it has reached the end of the sequence with
// <f OtxDrvIsIteratorEnd>.
// 
// An iterator is first initialized, and then repeatedly passed to one of the
// enumeration functions available. After an iterator has been initialized it can only
// be used to iterate over the same sequence, i.e. it can only be passed
// repeatedly to the same enum function with the same handle.
//
// To reuse an iterator for anouther enumeration it must be reinitialized with a
// call to <f OtxDrvIteratorBegin>
//
// The iterator could be checked for past-end iteration after each return from
// an enumeration function to stop enumeration when the last item has been examined.
// If a past-end iterator is passed to an enumeration function an error result code
// is returned.
//
//  @xref
//    <f OtxDrvEnumDeviceType>
//    <f OtxDrvEnumDevice>
//    <f OtxDrvIteratorBegin>
//    <f OtxDrvIsIteratorEnd>

#define OTX_ITERATOR    OTX_UINT32


/*******************************************************************************
   @enum
     Objects (devices, queues and pipes) can be queried about their
     attributes. Every object type supports different attributes but the
     values of each attribute is always one of these types.
*******************************************************************************/
enum OtxAttribTypeE {
  OTX_ATTRIB_SZ,     // @emem The attribute value is of type 'zero terminated string', i.e. an array of 8 bit char.
  OTX_ATTRIB_UINT32, // @emem The attribute is a 32 bit value.
  OTX_ATTRIB_BOOL,   // @emem The attribute is a 32 bit boolean value.
  OTX_ATTRIB_OCTETS  // @emem The attribute is an arbitrary collection of octets.
};

typedef enum OtxAttribTypeE OtxAttribTypeE;


/*******************************************************************************
   @type OTX_ATTRIB_ID |

     Objects (devices, queues and pipes) can be queried about their
     attributes. Every object type supports different attributes. Each
     attribute has an ID and a value. The IDs are defined in the scope of
     the particular object class and unique and meaningful only for the object
     which defines them. See relevant header files for actual IDs for object
     attributes.
*******************************************************************************/
#define OTX_ATTRIB_ID OTX_UINT32


/*******************************************************************************
  @enum 
    Common Device Attributes
*******************************************************************************/
enum OtxCommonDeviceAttributesE {
  OTX_ATTR_DEVICE_HOST   = 0,   // @emem (OTX_ATTRIB_UINT32) Read-only. 
                                // Handle of the host device
  OTX_ATTR_DEVICE_NIC    = 1,   // @emem (OTX_ATTRIB_UINT32) Read-only. 
                                // Handle of the NIC (board) 
                                // device hosting this device
  OTX_ATTR_DEVICE_TYPE   = 2,   // @emem (OTX_ATTRIB_UINT32) Read-only. 
                                // Type of the device (API Device Type)
  OTX_ATTR_DEVICE_NO     = 3,   // @emem (OTX_ATTRIB_UINT32) Read-only. 
                                // Device number
  OTX_ATTR_DEVICE_IRP_QUEUE_LIMIT = 4, // @emem (OTX_ATTRIB_UINT32) Read/Write. 
                                // Set (or get) the number of the IO Request Packets (IRP)
                                // that the driver can buffer for this device. Setting this
                                // attribute will also flush (clear) all pending IRPs for this device.
  OTX_ATTR_DEVICE_IRP_QUEUE_SIZE = 5, // @emem (OTX_ATTRIB_UINT32) Read-only. 
                                // Current number of IO Request Packets (IRP) in the IRP queue
                                // for this device.
  OTX_ATTR_DEVICE_SYNC_ENABLE = 6, // @emem (OTX_ATTRIB_UINT32) Read/Write. 
                                   // If set to any other value than 0xFFFFFFFF (-1), and if functionality
                                   // is supported by the device, the device will be enabled at the specified time
                                   // (device specific counter value). If the case of logical devices hosted by a DSP, the
                                   // device will be enabled (see <f OtxDrvEnable>) when the Frame Buffer Counter 
                                   // (<e OtxDspDeviceAttributesE.OTX_ATTR_DSP_FRAME_BUFFER_COUNTER>) reaches 
                                   // the specified value. This attribute can therefore be used to synchronize
                                   // the enabling moment for multiple devices.
                                
  OTX_ATTR_DEVICE_SYNC_DISABLE = 7, // @emem (OTX_ATTRIB_UINT32) Read/Write. 
                                   // If set to any other value than 0xFFFFFFFF (-1), and if functionality
                                   // is supported by the device, the device will be disabled at the specified time
                                   // (device specific counter value). If the case of logical devices hosted by a DSP, the
                                   // device will be disabled (see <f OtxDrvDisable>) when the Frame Buffer Counter 
                                   // (<e OtxDspDeviceAttributesE.OTX_ATTR_DSP_FRAME_BUFFER_COUNTER>) reaches 
                                   // the specified value. This attribute can therefore be used to synchronize
                                   // the disabling moment for multiple devices.
                                
  OTX_ATTR_END           = 8
};

typedef enum OtxCommonDeviceAttributesE OtxCommonDeviceAttributesE;


/*******************************************************************************
  @enum 
    Library Attributes
*******************************************************************************/
enum OtxLibraryAttributesE {
  OTX_ATTR_LIB_REV_STR              = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_SZ) Read-only. The Library revision as a string (E.g "1.0-P62" or "1.0" etc.)
  OTX_ATTR_LIB_REV_NUM              = OTX_ATTR_END +  1,  // @emem (OTX_ATTRIB_OCTETS) Read-only. The Library revision formatted as <t OtxVersionInfo>
  OTX_ATTR_LIB_FILE_NAME            = OTX_ATTR_END +  2,  // @emem (OTX_ATTRIB_SZ) Read-only. File name of the library
  OTX_ATTR_LIB_TYPE_STATIC          = OTX_ATTR_END +  3,  // @emem (OTX_ATTRIB_BOOL) Read-only. OTX_TRUE if the library is a static linked library, and OTX_FALSE if the library is a dynamic linked library
  OTX_ATTR_LIB_END                  = OTX_ATTR_END +  4 
};

typedef enum OtxLibraryAttributesE OtxLibraryAttributesE;


/*******************************************************************************
  @enum 
    Driver Device Attributes
*******************************************************************************/
enum OtxDriverDeviceAttributesE {
  OTX_ATTR_DRV_REV_STR                   = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_SZ) Read-only. The Driver revision as a string (E.g "1.0-P62" or "1.0" etc.)
  OTX_ATTR_DRV_REV_NUM                   = OTX_ATTR_END +  1,  // @emem (OTX_ATTRIB_OCTETS) Read-only. The Driver revision formatted as <t OtxVersionInfo>
  OTX_ATTR_DRV_TIMESTAMP_TICK_FREQUENCY  = OTX_ATTR_END +  2,  // @emem (OTX_ATTRIB_UINT32) Read-only. The timestamp tick frequency (in Hz)
  OTX_ATTR_DRV_TIMESTAMP_FROM_1601       = OTX_ATTR_END +  3,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE (default) the timestamp is generated in 100ns increments from January 1, 1601. If set to OTX_FALSE, the timestamp is generated using the finest grained running count available in the system (QueryPerformanceCounter() in Win32).
  OTX_ATTR_DRV_END                       = OTX_ATTR_END +  4 
};

typedef enum OtxDriverDeviceAttributesE OtxDriverDeviceAttributesE;


/*******************************************************************************
  @enum 
    CtBus Device Attributes
*******************************************************************************/
enum OtxCtBusDeviceAttributesE {
  OTX_ATTR_CTBUS_OTX_TIMESLOTS_PER_HWY     = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Number of timeslot to reserve for OTX pipe connections 
                                                                 // (on CT Bus Highway 0 through the value assigned to attribute <e OtxCtBusDeviceAttributesE.OTX_ATTR_CTBUS_OTX_HWYS>)
                                                                 // Default is <t OTX_PIPE_TIMESLOTS_MAX> (all timeslot).
  OTX_ATTR_CTBUS_OTX_HWYS                  = OTX_ATTR_END +  1,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Number of highways to reserve for OTX pipe connections. 
                                                                 // Default is <t OTX_H100_HIGHWAYS_MAX> (all highways).
  OTX_ATTR_CTBUS_END                       = OTX_ATTR_END +  2 
};

typedef enum OtxCtBusDeviceAttributesE OtxCtBusDeviceAttributesE;


/*******************************************************************************
  @struct 
    Interconnect information for a pipe (used by the <e OtxPipeDeviceAttributesE.OTX_ATTR_PIPE_CT_BUS_CONNECTIONS>)
*******************************************************************************/
struct OtxPipeCtBusConnectionS {
  OTX_UINT32 nCtBus;    // @field CT Bus number
  OTX_UINT32 nHighway;  // @field Highway number on the CT bus
  OTX_UINT32 nTimeslot; // @field Timeslot on the highway on the CT bus
};

typedef struct OtxPipeCtBusConnectionS OtxPipeCtBusConnectionS;


/*******************************************************************************
  @enum 
    Pipe Device Attributes
*******************************************************************************/
enum OtxPipeDeviceAttributesE {
  OTX_ATTR_PIPE_CAPACITY                   = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_UINT32) Read-only. Capacity of the pipe in kbps
  OTX_ATTR_PIPE_IN_CONNECTED               = OTX_ATTR_END +  1,  // @emem (OTX_ATTRIB_UINT32) Read-only. Handle of device connected to input, or OTX_INVALID_HANDLE_VALUE
  OTX_ATTR_PIPE_OUT_CONNECTED              = OTX_ATTR_END +  2,  // @emem (OTX_ATTRIB_UINT32) Read-only. Handle of device connected to output, or OTX_INVALID_HANDLE_VALUE
  OTX_ATTR_PIPE_IN_HIGHWAY                 = OTX_ATTR_END +  3,  // @emem (OTX_ATTRIB_UINT32) Read-only. Number of the highway used as input
  OTX_ATTR_PIPE_OUT_HIGHWAY                = OTX_ATTR_END +  4,  // @emem (OTX_ATTRIB_UINT32) Read-only. Number of the highway used as output
  OTX_ATTR_PIPE_IN_CONN_MASK               = OTX_ATTR_END +  5,  // @emem (OTX_ATTRIB_OCTETS) Read-only. Connection mask for input (how connected to "in" highway)
  OTX_ATTR_PIPE_OUT_CONN_MASK              = OTX_ATTR_END +  6,  // @emem (OTX_ATTRIB_OCTETS) Read-only. Connection mask for output (how connected to "out" highway)
  OTX_ATTR_PIPE_IN_CONN_MASK_SIZE          = OTX_ATTR_END +  7,  // @emem (OTX_ATTRIB_UINT32) Read-only. Size of the input connection mask (in octets)
  OTX_ATTR_PIPE_OUT_CONN_MASK_SIZE         = OTX_ATTR_END +  8,  // @emem (OTX_ATTRIB_UINT32) Read-only. Size of the output connection mask (in octets)
  OTX_ATTR_PIPE_CT_BUS_CONNECTIONS_COUNT   = OTX_ATTR_END +  9,  // @emem (OTX_ATTRIB_UINT32) Read-only. If this pipe connects to a CT bus, this attribute returns
                                                                 // the number timeslot connections that this pipe occupies on the CT bus. This attribute can be
                                                                 // used to determince the size of the array to supply to 
                                                                 // the <e OtxPipeDeviceAttributesE.OTX_ATTR_PIPE_CT_BUS_CONNECTIONS_DETAILS> attribute.
  OTX_ATTR_PIPE_CT_BUS_CONNECTIONS_DETAILS = OTX_ATTR_END + 10,  // @emem (OTX_ATTRIB_OCTETS) Read-only. If this pipe is connected to an H.100/H.110 CT bus and is the source in
                                                                 // such a connection, this attribute supplies the connection information for this pipe as an 
                                                                 // an array of <t OtxPipeCtBusConnectionS>. The size of the array to supply to this attribute
                                                                 // can be determined with the <e OtxPipeDeviceAttributesE.OTX_ATTR_PIPE_CT_BUS_CONNECTIONS_COUNT> attribute.
  OTX_ATTR_PIPE_AUTO_ENABLE                = OTX_ATTR_END + 11,  // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_TRUE.
                                                                 // <nl>If this attribute is set to OTX_TRIE the pipe is automatically enabled (i.e. it
                                                                 // will transfer data) when the pipe is connected through a call to <f OtxDrvConnectPipe>.
                                                                 // <nl>If set to OTX_FALSE data will flow through the pipe first when the pipe is enable
                                                                 // via a call to <f OtxDrvEnable>.
  OTX_ATTR_PIPE_END                        = OTX_ATTR_END + 12 
};

typedef enum OtxPipeDeviceAttributesE OtxPipeDeviceAttributesE;


/*******************************************************************************
  @enum 
    Event Queue Device Attributes
*******************************************************************************/
enum OtxEventQueueDeviceAttributesE {
  OTX_ATTR_QUEUE_EVENT        = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_UINT32) Read-only. Handle to associated OTX_EVENT
  OTX_ATTR_QUEUE_SIZE         = OTX_ATTR_END +  1,  // @emem (OTX_ATTRIB_UINT32) Read-only. Size of the event queue in number of events
  OTX_ATTR_QUEUE_USAGE        = OTX_ATTR_END +  2,  // @emem (OTX_ATTRIB_UINT32) Read-only. Number of events currently in queue (less than size)
  OTX_ATTR_QUEUE_MAX_USAGE    = OTX_ATTR_END +  3,  // @emem (OTX_ATTRIB_UINT32) Read-only. Max usage since this attribute last read
  OTX_ATTR_QUEUE_LOST_EVENTS  = OTX_ATTR_END +  4,  // @emem (OTX_ATTRIB_UINT32) Read-only. Number of lost events since this attribute last read
  OTX_ATTR_QUEUE_END          = OTX_ATTR_END +  5
};

typedef enum OtxEventQueueDeviceAttributesE OtxEventQueueDeviceAttributesE;


/*******************************************************************************
  @struct 
    Data recorded for an event
*******************************************************************************/
struct OtxEventDataS {
  OTX_DATETIME     m_nTimeStamp;        // @field Absolute time the event was recorded
  OSA_ENUM_DECL(OTX_UINT32) m_eDeviceType;   // @field Device Type with which the event is associated
  OTX_UINT32       m_nSourceId;         // @field Device Id of the device that the event is associated with (supplied by the user when the device for opened/created)
  OTX_HANDLE       m_hDevice;           // @field Handle to the Device with which the event is associated
  OTX_NOTIFY_CAUSE m_nCode;             // @field Device specific cause code indicating what happened. Each device types has it own set of event cause codes
  OTX_TASK_REF     m_nRequestId;        // @field If the message is for a previous user request this member holds the user assigned request ID
  OTX_UINT32       m_nParam;            // @field Optional parameter, usually number of bytes to be fetched
};

typedef struct OtxEventDataS OtxEventDataS;


/*******************************************************************************
   @enum
     State of a Hardware Device. Before a device has been configured, it's state
     is UNKNOWN. After configuration and call to function <f OtxDrvEnable> can
     put the device into WORKING state.  The driver can automatically block the
     device and change the state to BLOCKED_AUTO in a case of non-working device.
     The device can also be manually blocked with function <f OtxDrvDisable>
*******************************************************************************/
enum OtxDeviceStateE {
  OTX_DEVICE_STATE_UNKNOWN   = 0,     // @emem Device state is unknown, device has not been initialized
  OTX_DEVICE_WORKING         = 1,     // @emem Device is operational
  OTX_DEVICE_BLOCKED_AUTO    = 2,     // @emem Device is automatically blocked due to an error condition
  OTX_DEVICE_BLOCKED_MANUAL  = 3      // @emem Device is manually blocked
};

typedef enum OtxDeviceStateE OtxDeviceStateE;


/*******************************************************************************
   @enum
     End of a Pipe. A pipe is always one-directional serial bit stream having
     an input and an output. Two pipes are needed for a duplex connection between
     two devices.

   @ex A Pipe can be characterized as follows |
<nl>
<nl>                                     Control
<nl>                              User Handle (OTX_HANDLE)
<nl>                                        |
<nl>                                       \|/
<nl>                     +----------------------------------------+
<nl> OTX_PIPE_INPUT ---> |       Pipe with a set capacity         | ---> OTX_PIPE_OUTPUT
<nl>                     +----------------------------------------+
<nl>
<nl>
*******************************************************************************/
enum OtxPipeEndE {
  OTX_PIPE_INPUT   = 0,    // @emem Pipe end which receives data from a device
  OTX_PIPE_OUTPUT  = 1     // @emem Pipe end that delivers data to a device
};

typedef enum OtxPipeEndE OtxPipeEndE;


/*******************************************************************************
   @enum
     Model of a Physical or Logical Devices. Every Physical or Logical
     Device contain 4 interfaces: Data In, Data Out, Control, and Events.

   @ex A Device can be characterized as follows |
<nl>
<nl>                         Control
<nl>                 User Handle (OTX_HANDLE)
<nl>                            |
<nl>                           \|/
<nl>                +----------------------+
<nl>                |                      |
<nl>                |     A Physical       |
<nl>  DATA IN  ---> |         or           | ---> DATA OUT
<nl>                |     Logical Device   |
<nl>                |                      |
<nl>                +----------------------+
<nl>                            |
<nl>                           \|/
<nl>                        Events Out
<nl>
*******************************************************************************/
enum OtxDeviceInterfaceE {
  OTX_DEVICE_NONE    = -1,
  OTX_DEVICE_INPUT   = 0,
  OTX_DEVICE_OUTPUT  = 1,
  OTX_DEVICE_CONTROL = 2,
  OTX_DEVICE_EVENT   = 3
};

typedef enum OtxDeviceInterfaceE OtxDeviceInterfaceE;


/******************************************************************************
  @enum
    Generic Device Event Codes
******************************************************************************/
enum OtxDeviceEventCodesE {
  /* None */
  OTX_EC_END
};

typedef enum OtxDeviceEventCodesE OtxDeviceEventCodesE;


// @type OtxDeviceType | Generic type for either OtxPhysicalDeviceTypeE or OtxLogicalDeviceTypeE
typedef OTX_INT32 OtxDeviceType;


#ifdef __linux__
#define UNALIGNED 
#endif


#if defined(__linux__)
#pragma pack()
#elif defined(__BORLANDC__)
#pragma option pop
#else
#pragma pack(pop)
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998 - 2003


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif   // OTX_TYPE_H
